﻿using System;
using System.IO;
using System.Text;
using System.Collections.Generic;
using System.Linq;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using SRTSolutions.Elevate;

namespace CSharpCode
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class Snippets
    {
        [TestMethod]
        public void IntsAreMutable()
        {
            int number = 0;
            number++;

            Assert.AreEqual(1, number);
        }

        [TestMethod]
        public void ConstsAreImmutable()
        {
            const int number = 0;
            const int result = number + 1;

            Assert.AreEqual(1, result);
            Assert.AreEqual(0, number);
        }

        [TestMethod]
        public void StringsAreImmutable()
        {
            string stringValue = "world!";
            string result = stringValue.Insert(0, "hello ");

            Assert.AreEqual("world!", stringValue);
            Assert.AreEqual("hello world!", result);
        }

        [TestMethod]
        public void UninferredDictionary()
        {
            //Here, the type is explictily given
            Dictionary<string, string> dictionary = new Dictionary<string, string>();
        }

        [TestMethod]
        public void InferredDictionary()
        {
            //but here, the type is inferred
            var dictionary = new Dictionary<string, string>();
        }

        public MemoryStream GetStream()
        {
            var stream = new MemoryStream();
            var writer = new StreamWriter(stream);
            writer.WriteLine("line one");
            writer.WriteLine("line two");
            writer.WriteLine("line three");
            writer.Flush();
            stream.Position = 0;
            return stream;
        }

        [TestMethod]
        public void CausingASideEffect()
        {
            using (var reader = new StreamReader(GetStream()))
            {
                var line1 = reader.ReadLine();
                var line2 = reader.ReadLine();

                Assert.AreNotEqual(line1, line2);
            }
        }

        [TestMethod]
        public void ConditionalOperator()
        {
            var x = 2;
            string result = x % 2 == 0 ? "yes" : "no";

            Assert.AreEqual("yes", result);
        }

        private int Square(int value)
        {
            return value * value;
        }

        private bool IsEven(int value)
        {
            return value % 2 == 0;
        }

        [TestMethod]
        public void SquaringWithForLoops()
        {
            IList<int> values = 0.Through(10).ToList();

            IList<int> squaredValues = new List<int>();

            for (int i = 0; i < values.Count; i++)
            {
                squaredValues.Add(Square(values[i]));
            }

            List<int> expectedSquaredValues = new int[] { 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100 }.ToList();
            CollectionAssert.AreEqual(expectedSquaredValues, squaredValues.ToList());
        }

        [TestMethod]
        public void SquaringWithForeachLoops()
        {
            IList<int> values = 0.Through(10).ToList();

            //square a list
            IList<int> squaredValues = new List<int>();

            foreach (int value in values)
            {
                squaredValues.Add(Square(value));
            }

            List<int> expectedSquaredValues = new int[] { 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100 }.ToList();
            CollectionAssert.AreEqual(expectedSquaredValues, squaredValues.ToList());
        }

        [TestMethod]
        public void EvensWithForeachLoop()
        {
            IList<int> values = 0.Through(10).ToList();

            //filter out the even values in a list
            IList<int> evens = new List<int>();

            foreach (int value in values)
            {
                if (IsEven(value))
                {
                    evens.Add(value);
                }
            }

            List<int> expectedEvens = new int[] { 0, 2, 4, 6, 8, 10 }.ToList();
            CollectionAssert.AreEqual(expectedEvens, evens.ToList());
        }

        [TestMethod]
        public void SquaresOfEvensForeachLoop()
        {
            IEnumerable<int> values = new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };

            //take the square of the even values
            IList<int> results = new List<int>();

            foreach (int value in values)
            {
                if (IsEven(value))
                {
                    results.Add(Square(value));
                }
            }

            List<int> expected = new int[] { 0, 4, 16, 36, 64, 100 }.ToList();
            CollectionAssert.AreEqual(expected, results.ToList());
        }

        [TestMethod]
        public void SquaresOfEvensWithLinq()
        {
            IEnumerable<int> numbers = new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };

            var squareOfEvens =
                numbers
                .Where(IsEven)
                .Select(Square);

            List<int> expected = new int[] { 0, 4, 16, 36, 64, 100 }.ToList();
            CollectionAssert.AreEqual(expected, squareOfEvens.ToList());
        }

        [TestMethod]
        public void SquaresOfEvensWithQuery()
        {
            IEnumerable<int> numbers = new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };

            var squareOfEvens = from number in numbers
                                where IsEven(number)
                                select Square(number);

            List<int> expected = new int[] { 0, 4, 16, 36, 64, 100 }.ToList();
            CollectionAssert.AreEqual(expected, squareOfEvens.ToList());
        }
    }
}
